;Tutorial 13.1 - Nigel Goodwin 2006
;Matrix LED display

	LIST	p=16F876		;tell assembler what chip we are using
	include "P16F876.inc"		;include the defaults for the chip
	ERRORLEVEL	0,	-302	;suppress bank selection messages
	__config 0x393A			;sets the configuration settings (oscillator type etc.)




		cblock	0x20			;start of general purpose registers
			count			;used in looping routines
			count1			;used in delay routine
			counta			;used in delay routine
			countb			;used in delay routine
			tmp1			;temporary storage
			tmp2
			p_temp			;save register
			s_temp			;save register
			w_temp			;save register
						
						;registers for display

			row_pos			;position on row

			zero			;zero storage
			one			;one storage
			two			;two storage
			three			;three storage
			four			;four storage
			five			;five storage
			six			;six storage
			seven			;seven storage

		endc

ROW_PORT	Equ	PORTB			;row port
ROW_TRIS	Equ	TRISB

COL_PORT	Equ	PORTC			;column port
COL_TRIS	Equ	TRISC


		org	0x0000
		goto	START

;**************************************************************************	
;			     	Interrupt routine
;**************************************************************************



; 	Interrupt routine handles TMR2 which generates a 1ms tick

;	Interrupt vector

		ORG	0x0004


INT
		movwf	w_temp		; Save W register
		swapf	STATUS,W	; Swap status to be saved into W
		movwf	s_temp		; Save STATUS register
		movfw	PCLATH
		movwf	p_temp		; Save PCLATH 
	
		btfss	PIR1,TMR2IF	; Flag set if TMR2 interrupt
		goto	INTX		; Jump if not timed out

		; Timer (TMR2) timeout
	

		bcf	PIR1,TMR2IF	; Clear the calling flag


		btfss	row_pos, 0	;check which ROW was last
		goto	Do_One
		btfss	row_pos, 1	;check which ROW was last
		goto	Do_Two
		btfss	row_pos, 2	;check which ROW was last
		goto	Do_Three
		btfss	row_pos, 3	;check which ROW was last
		goto	Do_Four
		btfss	row_pos, 4	;check which ROW was last
		goto	Do_Five
		btfss	row_pos, 5	;check which ROW was last
		goto	Do_Six
		btfss	row_pos, 6	;check which ROW was last
		goto	Do_Seven

Do_Zero		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	zero,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 0
		bcf	ROW_PORT, 0	;turn ON row zero
		goto	INTX

Do_One		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	one,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 1
		bcf	ROW_PORT, 1	;turn ON row one
		goto	INTX

Do_Two		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	two,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 2
		bcf	ROW_PORT, 2	;turn ON row two
		goto	INTX

Do_Three	movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	three,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 3
		bcf	ROW_PORT, 3	;turn ON row three
		goto	INTX

Do_Four		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	four,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 4
		bcf	ROW_PORT, 4	;turn ON row four
		goto	INTX

Do_Five		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	five,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 5
		bcf	ROW_PORT, 5	;turn ON row five
		goto	INTX

Do_Six		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	six,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 6
		bcf	ROW_PORT, 6	;turn ON row six
		goto	INTX

Do_Seven	movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	seven,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 7
		bcf	ROW_PORT, 7	;turn ON row seven

INTX
		movfw	p_temp
		movwf	PCLATH		; Restore PCLATH
		swapf	s_temp,W
		movwf	STATUS		; Restore STATUS register - restores bank
		swapf	w_temp,F
		swapf	w_temp,W	; Restore W register
		retfie


;program initialisation

START		
		BANKSEL ADCON1			;disable analogue inputs
		movlw 	0x06
		movwf 	ADCON1
		BANKSEL PORTA
		bsf 	STATUS,		RP0	;select bank 1
		movlw	b'00000000'		;Set port data directions, data output
		movwf	ROW_TRIS
		movwf	COL_TRIS
		bcf 	STATUS,		RP0	;select bank 0

		clrf	COL_PORT		;turn OFF all LED's
		movlw	0xFF
		movwf	ROW_PORT	

		movlw	0x01
		movwf	row_pos


		;	Set up Timer 2.
	
		;movlw	b'01111110'		; Post scale /16, pre scale /16, TMR2 ON
		;uncomment previous line, and comment next line, to slow multiplexing speed
		;so you can see the multiplexing happening
		movlw	b'00010110'		; Post scale /4, pre scale /16, TMR2 ON
		movwf	T2CON

		bsf 	STATUS,		RP0	;select bank 1

		movlw	.249			; Set up comparator
		movwf	PR2

		bsf	PIE1,TMR2IE		; Enable TMR2 interrupt

		bcf 	STATUS,		RP0	;select bank 0

		; Global interrupt enable

		bsf	INTCON,PEIE		; Enable all peripheral interrupts
		bsf	INTCON,GIE		; Global interrupt enable

		bcf 	STATUS,		RP0	;select bank 0
	

;main program loop

		
Main		call	Square			;dislay a square
		call	Delay1Sec
		call	Invert			;invert it
		call	Delay1Sec
		goto	Main			;endless loop



;start of subroutines

Square		movlw	0xFF
		movwf	zero
		movlw	0x81
		movwf	one
		movlw	0x81
		movwf	two
		movlw	0x81
		movwf	three
		movlw	0x81
		movwf	four
		movlw	0x81
		movwf	five
		movlw	0x81
		movwf	six
		movlw	0xFF
		movwf	seven
		return

Invert		movlw	0xFF
		xorwf	zero, f
		xorwf	one, f
		xorwf	two, f
		xorwf	three, f
		xorwf	four, f
		xorwf	five, f
		xorwf	six, f
		xorwf	seven, f
		return

Delay1Sec	call	Delay250
		call	Delay250
		call	Delay250
		call	Delay250
		Return

Delay255	movlw	0xff	;delay 255mS
		goto	d0
Delay250	movlw	d'250'	;delay 250mS
		goto	d0
Delay100	movlw	d'100'	;delay 100mS
		goto	d0
Delay50		movlw	d'50'	;delay 50mS
		goto	d0
Delay20		movlw	d'20'	;delay 20mS
		goto	d0
Delay10		movlw	d'10'	;delay 10mS
		goto	d0
Delay1		movlw	d'1'	;delay 1mS
		goto	d0
Delay5		movlw	0x05	;delay 5ms
d0		movwf	count1
d1		movlw	0xE7
		movwf	counta
		movlw	0x04
		movwf	countb
Delay_0		decfsz	counta, f
		goto	$+2
		decfsz	countb, f
		goto	Delay_0
		decfsz	count1	,f
		goto	d1
		retlw	0x00


	END
