#include <pic.h>				// pic specific identifiers
#define _XTAL_FREQ  20000000	// Xtal speed
__CONFIG(0x37F2);				// Config bits

#define ROW_PORT		PORTB			//
#define ROW_TRIS		TRISB
#define COL_PORT		PORTC			//
#define COL_TRIS		TRISC

	// Required prototypes
void DelayMs(int x), square(void), invert(void);

	// Variables	
unsigned char pos;				// keep tabs on rows
unsigned char leds[8];			// led buffer
unsigned char pow[] = {1,2,4,8,16,32,64,128}; // power array	

void main(void)							// program entry
	{
	ADCON1 = 0x6;
	ROW_TRIS = 0b00000000;				// row port as outputs
	COL_TRIS = 0b00000000;				// col port as outputs
	T2CON = 0x16;						// setup timer 2
	ROW_PORT = 0;
	COL_PORT = 0xFF;					// setup led array
	PR2 = 249;							// timer preload value
	TMR2IE = 1;							// enable timer 2 interrupt
	PEIE = 1;							// enable peripheral interrupt
	GIE = 1;							// enableglobal interrupt
	pos = 0;							// start at beginning
	while(1)							// endless Loop
		{
		square();						// fill array buffer
		DelayMs(1000);					// watch display
		invert();						// invert buffer
		DelayMs(1000);					// watch display
		}
	}

void interrupt ISR()
	{
	if(TMR2IF)			// omly do if timeout occured
		{
		ROW_PORT = 0xff;
		COL_PORT = leds[pos];			// Retrive current array
		ROW_PORT -= pow[pos];			// Keep powers in table
		if(pos++ == 7) pos = 0;			// cycle through array buffer
		TMR2IF = 0;						// clear interrupt
		}	
	}

void square()
	{	// initial with square bitmap
	leds[0] = 0xFF; leds[1] = 0x81; leds[2] = 0x81; leds[3] = 0x81;
	leds[4] = 0x81; leds[5] = 0x81; leds[6] = 0x81; leds[7] = 0xFF;
	}

void invert()
	{	// this just inverts the square bitmap.
	char x;
	for(x=0;x<8;x++)
		leds[x] ^= 0xFF;
	}
	
void DelayMs(int x)
	{	// wrapper function for larger delays.
	while(x--)
		__delay_ms(1);
	}