;Tutorial 13.2 - Nigel Goodwin 2006
;Matrix LED display

	LIST	p=16F876		;tell assembler what chip we are using
	include "P16F876.inc"		;include the defaults for the chip
	ERRORLEVEL	0,	-302	;suppress bank selection messages
	__config 0x393A			;sets the configuration settings (oscillator type etc.)




		cblock	0x20			;start of general purpose registers
			count			;used in looping routines
			count1			;used in delay routine
			counta			;used in delay routine
			countb			;used in delay routine
			tmp1			;temporary storage
			tmp2
			p_temp			;save register
			s_temp			;save register
			w_temp			;save register
						
						;registers for display

			row_pos			;position on row

			zero			;zero storage
			one			;one storage
			two			;two storage
			three			;three storage
			four			;four storage
			five			;five storage
			six			;six storage
			seven			;seven storage

			zero1			;zero storage
			one1			;one storage
			two1			;two storage
			three1			;three storage
			four1			;four storage
			five1			;five storage
			six1			;six storage
			seven1			;seven storage

		endc

ROW_PORT	Equ	PORTB			;row port
ROW_TRIS	Equ	TRISB

COL_PORT	Equ	PORTC			;column port
COL_TRIS	Equ	TRISC


		org	0x0000
		goto	START

;**************************************************************************	
;			     	Interrupt routine
;**************************************************************************



; 	Interrupt routine handles TMR2 which generates a 1ms tick

;	Interrupt vector

		ORG	0x0004


INT
		movwf	w_temp		; Save W register
		swapf	STATUS,W	; Swap status to be saved into W
		movwf	s_temp		; Save STATUS register
		movfw	PCLATH
		movwf	p_temp		; Save PCLATH 
	
		btfss	PIR1,TMR2IF	; Flag set if TMR2 interrupt
		goto	INTX		; Jump if not timed out

		; Timer (TMR2) timeout
	

		bcf	PIR1,TMR2IF	; Clear the calling flag


		btfss	row_pos, 0	;check which ROW was last
		goto	Do_One
		btfss	row_pos, 1	;check which ROW was last
		goto	Do_Two
		btfss	row_pos, 2	;check which ROW was last
		goto	Do_Three
		btfss	row_pos, 3	;check which ROW was last
		goto	Do_Four
		btfss	row_pos, 4	;check which ROW was last
		goto	Do_Five
		btfss	row_pos, 5	;check which ROW was last
		goto	Do_Six
		btfss	row_pos, 6	;check which ROW was last
		goto	Do_Seven

Do_Zero		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	zero,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 0
		bcf	ROW_PORT, 0	;turn ON row zero
		goto	INTX

Do_One		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	one,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 1
		bcf	ROW_PORT, 1	;turn ON row one
		goto	INTX

Do_Two		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	two,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 2
		bcf	ROW_PORT, 2	;turn ON row two
		goto	INTX

Do_Three	movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	three,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 3
		bcf	ROW_PORT, 3	;turn ON row three
		goto	INTX

Do_Four		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	four,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 4
		bcf	ROW_PORT, 4	;turn ON row four
		goto	INTX

Do_Five		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	five,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 5
		bcf	ROW_PORT, 5	;turn ON row five
		goto	INTX

Do_Six		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	six,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 6
		bcf	ROW_PORT, 6	;turn ON row six
		goto	INTX

Do_Seven	movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	seven,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 7
		bcf	ROW_PORT, 7	;turn ON row seven

INTX
		movfw	p_temp
		movwf	PCLATH		; Restore PCLATH
		swapf	s_temp,W
		movwf	STATUS		; Restore STATUS register - restores bank
		swapf	w_temp,F
		swapf	w_temp,W	; Restore W register
		retfie

;program initialisation

START		
		BANKSEL ADCON1			;disable analogue inputs
		movlw 	0x06
		movwf 	ADCON1
		BANKSEL PORTA
		bsf 	STATUS,		RP0	;select bank 1
		movlw	b'00000000'		;Set port data directions, data output
		movwf	ROW_TRIS
		movwf	COL_TRIS
		bcf 	STATUS,		RP0	;select bank 0

		clrf	COL_PORT		;turn OFF all LED's
		movlw	0xFF
		movwf	ROW_PORT	

		movlw	0x01
		movwf	row_pos

		call	Clear			;clear display registers

		;	Set up Timer 2.
	
		;movlw	b'01111110'		; Post scale /16, pre scale /16, TMR2 ON
		;uncomment previous line, and comment next line, to slow multiplexing speed
		;so you can see the multiplexing happening
		movlw	b'00010110'		; Post scale /4, pre scale /16, TMR2 ON
		movwf	T2CON

		bsf 	STATUS,		RP0	;select bank 1

		movlw	.249			; Set up comparator
		movwf	PR2

		bsf	PIE1,TMR2IE		; Enable TMR2 interrupt

		bcf 	STATUS,		RP0	;select bank 0

		; Global interrupt enable

		bsf	INTCON,PEIE		; Enable all peripheral interrupts
		bsf	INTCON,GIE		; Global interrupt enable

		bcf 	STATUS,		RP0	;select bank 0
	

;main program loop

		
Main		
		call	Load_Zero		;scroll right
		call	ScrollR8
		call	Load_One
		call	ScrollR8	
		call	Load_Two
		call	ScrollR8
		call	Load_Three
		call	ScrollR8	
		call	Load_Four
		call	ScrollR8
		call	Load_Five
		call	ScrollR8
		call	Load_Six
		call	ScrollR8
		call	Load_Seven
		call	ScrollR8
		call	Load_Eight
		call	ScrollR8
		call	Load_Nine
		call	ScrollR8
		call	Delay1Sec

		call	Clear
		call	Load_Zero		;scroll left
		call	ScrollL8
		call	Load_One
		call	ScrollL8	
		call	Load_Two
		call	ScrollL8
		call	Load_Three
		call	ScrollL8	
		call	Load_Four
		call	ScrollL8
		call	Load_Five
		call	ScrollL8
		call	Load_Six
		call	ScrollL8
		call	Load_Seven
		call	ScrollL8
		call	Load_Eight
		call	ScrollL8
		call	Load_Nine
		call	ScrollL8
		call	Delay1Sec

		call	Clear
		call	Load_Zero		;scroll up
		call	ScrollU8
		call	Load_One
		call	ScrollU8	
		call	Load_Two
		call	ScrollU8
		call	Load_Three
		call	ScrollU8	
		call	Load_Four
		call	ScrollU8
		call	Load_Five
		call	ScrollU8
		call	Load_Six
		call	ScrollU8
		call	Load_Seven
		call	ScrollU8
		call	Load_Eight
		call	ScrollU8
		call	Load_Nine
		call	ScrollU8
		call	Delay1Sec

		call	Clear
		call	Load_Zero		;scroll down
		call	ScrollD8
		call	Load_One
		call	ScrollD8	
		call	Load_Two
		call	ScrollD8
		call	Load_Three
		call	ScrollD8	
		call	Load_Four
		call	ScrollD8
		call	Load_Five
		call	ScrollD8
		call	Load_Six
		call	ScrollD8
		call	Load_Seven
		call	ScrollD8
		call	Load_Eight
		call	ScrollD8
		call	Load_Nine
		call	ScrollD8	
		call	Delay1Sec

		goto	Main			;endless loop



;start of subroutines

Clear		clrf	zero			;clear display storage
		clrf	one
		clrf	two
		clrf	three
		clrf	four
		clrf	five
		clrf	six
		clrf	seven
		return

Clear1		clrf	zero1			;clear data storage
		clrf	one1
		clrf	two1
		clrf	three1
		clrf	four1
		clrf	five1
		clrf	six1
		clrf	seven1
		return

Square		movlw	0xFF
		movwf	zero
		movlw	0x81
		movwf	one
		movlw	0x81
		movwf	two
		movlw	0x81
		movwf	three
		movlw	0x81
		movwf	four
		movlw	0x81
		movwf	five
		movlw	0x81
		movwf	six
		movlw	0xFF
		movwf	seven
		return

Block1		movlw	0xFF
		movwf	zero1
		movlw	0xFF
		movwf	one1
		movlw	0xFF
		movwf	two1
		movlw	0xFF
		movwf	three1
		movlw	0xFF
		movwf	four1
		movlw	0xFF
		movwf	five1
		movlw	0xFF
		movwf	six1
		movlw	0xFF
		movwf	seven1
		return

Load_Zero	movlw	0x00
		movwf	zero1
		movlw	0x38
		movwf	one1
		movlw	0x44
		movwf	two1
		movlw	0x4C
		movwf	three1
		movlw	0x54
		movwf	four1
		movlw	0x64
		movwf	five1
		movlw	0x44
		movwf	six1
		movlw	0x38
		movwf	seven1
		return

Load_One	movlw	0x00
		movwf	zero1
		movlw	0x38
		movwf	one1
		movlw	0x10
		movwf	two1
		movlw	0x10
		movwf	three1
		movlw	0x10
		movwf	four1
		movlw	0x10
		movwf	five1
		movlw	0x30
		movwf	six1
		movlw	0x10
		movwf	seven1
		return

Load_Two	movlw	0x00
		movwf	zero1
		movlw	0x7C
		movwf	one1
		movlw	0x20
		movwf	two1
		movlw	0x10
		movwf	three1
		movlw	0x08
		movwf	four1
		movlw	0x04
		movwf	five1
		movlw	0x44
		movwf	six1
		movlw	0x38
		movwf	seven1
		return

Load_Three	movlw	0x00
		movwf	zero1
		movlw	0x38
		movwf	one1
		movlw	0x44
		movwf	two1
		movlw	0x04
		movwf	three1
		movlw	0x08
		movwf	four1
		movlw	0x10
		movwf	five1
		movlw	0x08
		movwf	six1
		movlw	0x7C
		movwf	seven1
		return

Load_Four	movlw	0x00
		movwf	zero1
		movlw	0x08
		movwf	one1
		movlw	0x08
		movwf	two1
		movlw	0x7C
		movwf	three1
		movlw	0x48
		movwf	four1
		movlw	0x28
		movwf	five1
		movlw	0x18
		movwf	six1
		movlw	0x08
		movwf	seven1
		return

Load_Five	movlw	0x00
		movwf	zero1
		movlw	0x38
		movwf	one1
		movlw	0x44
		movwf	two1
		movlw	0x04
		movwf	three1
		movlw	0x04
		movwf	four1
		movlw	0x78
		movwf	five1
		movlw	0x40
		movwf	six1
		movlw	0x7C
		movwf	seven1
		return

Load_Six	movlw	0x00
		movwf	zero1
		movlw	0x38
		movwf	one1
		movlw	0x44
		movwf	two1
		movlw	0x44
		movwf	three1
		movlw	0x78
		movwf	four1
		movlw	0x40
		movwf	five1
		movlw	0x20
		movwf	six1
		movlw	0x18
		movwf	seven1
		return

Load_Seven	movlw	0x00
		movwf	zero1
		movlw	0x20
		movwf	one1
		movlw	0x20
		movwf	two1
		movlw	0x20
		movwf	three1
		movlw	0x10
		movwf	four1
		movlw	0x08
		movwf	five1
		movlw	0x04
		movwf	six1
		movlw	0x7C
		movwf	seven1
		return

Load_Eight	movlw	0x00
		movwf	zero1
		movlw	0x38
		movwf	one1
		movlw	0x44
		movwf	two1
		movlw	0x44
		movwf	three1
		movlw	0x38
		movwf	four1
		movlw	0x44
		movwf	five1
		movlw	0x44
		movwf	six1
		movlw	0x38
		movwf	seven1
		return

Load_Nine	movlw	0x00
		movwf	zero1
		movlw	0x30
		movwf	one1
		movlw	0x08
		movwf	two1
		movlw	0x04
		movwf	three1
		movlw	0x3C
		movwf	four1
		movlw	0x44
		movwf	five1
		movlw	0x44
		movwf	six1
		movlw	0x38
		movwf	seven1
		return

ScrollR8	call	ScrollR
		call	Delay50	
		call	ScrollR
		call	Delay50	
		call	ScrollR
		call	Delay50	
		call	ScrollR
		call	Delay50	
		call	ScrollR
		call	Delay50	
		call	ScrollR
		call	Delay50	
		call	ScrollR
		call	Delay50	
		call	ScrollR
		call	Delay50
		return

ScrollL8	call	ScrollL
		call	Delay50	
		call	ScrollL
		call	Delay50	
		call	ScrollL
		call	Delay50	
		call	ScrollL
		call	Delay50	
		call	ScrollL
		call	Delay50	
		call	ScrollL
		call	Delay50	
		call	ScrollL
		call	Delay50	
		call	ScrollL
		call	Delay50
		return

ScrollU8	call	ScrollUp
		call	Delay50	
		call	ScrollUp
		call	Delay50	
		call	ScrollUp
		call	Delay50	
		call	ScrollUp
		call	Delay50	
		call	ScrollUp
		call	Delay50	
		call	ScrollUp
		call	Delay50	
		call	ScrollUp
		call	Delay50	
		call	ScrollUp
		call	Delay50
		return

ScrollD8	call	ScrollDn
		call	Delay50	
		call	ScrollDn
		call	Delay50	
		call	ScrollDn
		call	Delay50	
		call	ScrollDn
		call	Delay50	
		call	ScrollDn
		call	Delay50	
		call	ScrollDn
		call	Delay50	
		call	ScrollDn
		call	Delay50	
		call	ScrollDn
		call	Delay50
		return

ScrollUp	movf	six, w
		movwf	seven
		movf	five, w
		movwf	six
		movf	four, w
		movwf	five
		movf	three, w
		movwf	four
		movf	two, w
		movwf	three
		movf	one, w
		movwf	two
		movf	zero, w
		movwf	one
		movf	seven1, w
		movwf	zero
		movf	six1, w
		movwf	seven1
		movf	five1, w
		movwf	six1
		movf	four1, w
		movwf	five1
		movf	three1, w
		movwf	four1
		movf	two1, w
		movwf	three1
		movf	one1, w
		movwf	two1
		movf	zero1, w
		movwf	one1
		movlw	0x00
		movwf	zero1
		return

ScrollDn	movf	one, w
		movwf	zero
		movf	two, w
		movwf	one
		movf	three, w
		movwf	two
		movf	four, w
		movwf	three
		movf	five, w
		movwf	four
		movf	six, w
		movwf	five
		movf	seven, w
		movwf	six
		movf	zero1, w
		movwf	seven
		movf	one1, w
		movwf	zero1
		movf	two1, w
		movwf	one1
		movf	three1, w
		movwf	two1
		movf	four1, w
		movwf	three1
		movf	five1, w
		movwf	four1
		movf	six1, w
		movwf	five1
		movf	seven1, w
		movwf	six1
		movlw	0x00
		movwf	seven1
		return

ScrollL		bcf	STATUS, C
		rlf	zero1, f
		rlf	zero, f
		bcf	STATUS, C
		rlf	one1, f
		rlf	one, f
		bcf	STATUS, C
		rlf	two1, f
		rlf	two, f
		bcf	STATUS, C
		rlf	three1, f
		rlf	three, f
		bcf	STATUS, C
		rlf	four1, f
		rlf	four, f
		bcf	STATUS, C
		rlf	five1, f
		rlf	five, f
		bcf	STATUS, C
		rlf	six1, f
		rlf	six, f
		bcf	STATUS, C
		rlf	seven1, f
		rlf	seven, f
		return

ScrollR		bcf	STATUS, C
		rrf	zero1, f
		rrf	zero, f
		bcf	STATUS, C
		rrf	one1, f
		rrf	one, f
		bcf	STATUS, C
		rrf	two1, f
		rrf	two, f
		bcf	STATUS, C
		rrf	three1, f
		rrf	three, f
		bcf	STATUS, C
		rrf	four1, f
		rrf	four, f
		bcf	STATUS, C
		rrf	five1, f
		rrf	five, f
		bcf	STATUS, C
		rrf	six1, f
		rrf	six, f
		bcf	STATUS, C
		rrf	seven1, f
		rrf	seven, f
		return

Invert		movlw	0xFF
		xorwf	zero, f
		xorwf	one, f
		xorwf	two, f
		xorwf	three, f
		xorwf	four, f
		xorwf	five, f
		xorwf	six, f
		xorwf	seven, f
		return

Invert1		movlw	0xFF
		xorwf	zero1, f
		xorwf	one1, f
		xorwf	two1, f
		xorwf	three1, f
		xorwf	four1, f
		xorwf	five1, f
		xorwf	six1, f
		xorwf	seven1, f
		return

Swap		movf	zero1, w
		movwf	zero
		movf	one1, w
		movwf	one
		movf	two1, w
		movwf	two
		movf	three1, w
		movwf	three
		movf	four1, w
		movwf	four
		movf	five1, w
		movwf	five
		movf	six1, w
		movwf	six
		movf	seven1, w
		movwf	seven
		return

Delay1Sec	call	Delay250
		call	Delay250
		call	Delay250
		call	Delay250
		Return


Delay255	movlw	0xff	;delay 255mS
		goto	d0
Delay250	movlw	d'250'	;delay 250mS
		goto	d0
Delay100	movlw	d'100'	;delay 100mS
		goto	d0
Delay50		movlw	d'50'	;delay 50mS
		goto	d0
Delay20		movlw	d'20'	;delay 20mS
		goto	d0
Delay10		movlw	d'10'	;delay 10mS
		goto	d0
Delay1		movlw	d'1'	;delay 1mS
		goto	d0
Delay5		movlw	0x05	;delay 5ms
d0		movwf	count1
d1		movlw	0xE7
		movwf	counta
		movlw	0x04
		movwf	countb
Delay_0		decfsz	counta, f
		goto	$+2
		decfsz	countb, f
		goto	Delay_0
		decfsz	count1	,f
		goto	d1
		retlw	0x00


	END
