#include <pic.h>				// pic specific identifiers
#define _XTAL_FREQ  20000000	// Xtal speed
__CONFIG(0x37F2);				// Config bits

#define ROW_PORT		PORTB			//
#define ROW_TRIS		TRISB
#define COL_PORT		PORTC			//
#define COL_TRIS		TRISC

extern const unsigned char fnt[];   // delclare as external (in another file)

	// Required prototypes
void DelayMs(int x), scroll(char dir), invert(void);
void clear(void), display(const char* str);

	// Variables
unsigned char pos;				// keep tabs on rows
unsigned char leds[8];			// led buffer
unsigned char leds2[8];			// 2nd led buffer
unsigned char pow[] = {1,2,4,8,16,32,64,128}; // power array

void main(void)							// program entry
	{
	char x,y,z,t;
	ADCON1 = 0x6;
	ROW_TRIS = 0b00000000;				// row port as outputs
	COL_TRIS = 0b00000000;				// col port as outputs
	T2CON = 0x16;						// setup timer 2
	ROW_PORT = 0;
	COL_PORT = 0xFF;					// setup led array
	PR2 = 249;							// timer preload value
	TMR2IE = 1;							// enable timer 2 interrupt
	PEIE = 1;							// enable peripheral interrupt
	GIE = 1;							// enableglobal interrupt
	pos = 0;							// start at beginning
	while(1)							// endless Loop
		{
		display("Hello world!!");		// display a message
		clear();						// start again
		DelayMs(1000);
		}
	}

void display(const char* str)
	{
	int addr;							// large table (requires an int)
	int z;								// looping variable
	while(*str!=0)
		{
		addr = ((int)*str++ - 0x20);	// locate character in table
		addr *= 8;						// 8 bytes in each char
		for(z=0;z<8;z++)
			leds2[z] = fnt[addr+z];		// load em up!!
		for(z=0;z<8;z++)
			scroll(1);					// scroll right.
		}
	}

void interrupt ISR()
	{
	if(TMR2IF)			// omly do if timeout occured
		{
		ROW_PORT = 0xff;
		COL_PORT = leds[pos];			// Retrive current array
		ROW_PORT -= pow[pos];			// Keep powers in table
		if(pos++ == 7) pos = 0;			// cycle through array buffer
		TMR2IF = 0;						// clear interrupt
		}
	}

void clear()		// clear buffer
	{
	char x;
	for(x=0;x<8;x++)
		leds[x] = 0;

	}
void scroll(char dir)
	{
	char x;
	switch (dir)
		{
		case 0:					// scroll left
			for(x=0;x<8;x++)
				{
				if(leds2[x] & 1)
					leds[x]+=128;
				leds[x] >>= 1;
				leds2[x]>>= 1;
				}
			DelayMs(100);
			break;
		case 1:					// scroll right
			for(x=0;x<8;x++)
				{
				if(leds2[x] & 128)
					leds[x]+=1;
				leds[x] <<= 1;
				leds2[x] <<= 1;
				}
			DelayMs(100);
			break;
		case 2:					// scroll up
			for(x=7;x>0;x--)
				leds[x] = leds[x-1];
			leds[0] = leds2[7];
			for(x=7;x>0;x--)
				leds2[x] = leds2[x-1];
			leds2[0] = 0;
			DelayMs(100);
			break;
		case 3:			 		// scroll down
			for(x=0;x<8;x++)
				leds[x] = leds[x+1];
			leds2[7] = leds[0];
			for(x=0;x<8;x++)
				leds2[x] = leds2[x+1];
			leds2[7] = 0;
			DelayMs(100);
		}

	}

void DelayMs(int x)
	{	// wrapper function for larger delays.
	while(x--)
		__delay_ms(1);
	}