;Tutorial 13.4 - Nigel Goodwin 2006
;Matrix LED display - display a scrolling text message

	LIST	p=16F876		;tell assembler what chip we are using
	include "P16F876.inc"		;include the defaults for the chip
	ERRORLEVEL	0,	-302	;suppress bank selection messages
	__config 0x393A			;sets the configuration settings (oscillator type etc.)




		cblock	0x20			;start of general purpose registers
			count			;used in looping routines
			count1			;used in delay routine
			counta			;used in delay routine
			countb			;used in delay routine
			tmp1			;temporary storage
			tmp2
			p_temp			;save register
			s_temp			;save register
			w_temp			;save register

			offsetL			;table registers
			offsetH
			offsetL1		
			offsetH1						
						;registers for display

			row_pos			;position on row

			zero			;zero storage
			one			;one storage
			two			;two storage
			three			;three storage
			four			;four storage
			five			;five storage
			six			;six storage
			seven			;seven storage

			zero1			;zero storage
			one1			;one storage
			two1			;two storage
			three1			;three storage
			four1			;four storage
			five1			;five storage
			six1			;six storage
			seven1			;seven storage

		endc

ROW_PORT	Equ	PORTB			;row port
ROW_TRIS	Equ	TRISB

COL_PORT	Equ	PORTC			;column port
COL_TRIS	Equ	TRISC


		org	0x0000
		goto	START

;**************************************************************************	
;			     	Interrupt routine
;**************************************************************************



; 	Interrupt routine handles TMR2 which generates a 1ms tick

;	Interrupt vector

		ORG	0x0004


INT
		movwf	w_temp		; Save W register
		swapf	STATUS,W	; Swap status to be saved into W
		movwf	s_temp		; Save STATUS register
		movfw	PCLATH
		movwf	p_temp		; Save PCLATH 
	
		btfss	PIR1,TMR2IF	; Flag set if TMR2 interrupt
		goto	INTX		; Jump if not timed out

		; Timer (TMR2) timeout
	

		bcf	PIR1,TMR2IF	; Clear the calling flag


		btfss	row_pos, 0	;check which ROW was last
		goto	Do_One
		btfss	row_pos, 1	;check which ROW was last
		goto	Do_Two
		btfss	row_pos, 2	;check which ROW was last
		goto	Do_Three
		btfss	row_pos, 3	;check which ROW was last
		goto	Do_Four
		btfss	row_pos, 4	;check which ROW was last
		goto	Do_Five
		btfss	row_pos, 5	;check which ROW was last
		goto	Do_Six
		btfss	row_pos, 6	;check which ROW was last
		goto	Do_Seven
		btfss	row_pos, 7	;check which ROW was last
		goto	Do_Zero

Do_Zero		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	zero,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 0
		bcf	ROW_PORT, 0	;turn ON row zero
		goto	INTX

Do_One		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	one,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 1
		bcf	ROW_PORT, 1	;turn ON row one
		goto	INTX

Do_Two		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	two,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 2
		bcf	ROW_PORT, 2	;turn ON row two
		goto	INTX

Do_Three	movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	three,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 3
		bcf	ROW_PORT, 3	;turn ON row three
		goto	INTX

Do_Four		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	four,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 4
		bcf	ROW_PORT, 4	;turn ON row four
		goto	INTX

Do_Five		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	five,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 5
		bcf	ROW_PORT, 5	;turn ON row five
		goto	INTX

Do_Six		movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	six,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 6
		bcf	ROW_PORT, 6	;turn ON row six
		goto	INTX

Do_Seven	movlw	0xFF
		movwf	ROW_PORT	;turn off all rows
		movwf	row_pos
		movf	seven,	w
		movwf	COL_PORT	;load columns
		bcf	row_pos, 7
		bcf	ROW_PORT, 7	;turn ON row seven
		goto	INTX

INTX
		movfw	p_temp
		movwf	PCLATH		; Restore PCLATH
		swapf	s_temp,W
		movwf	STATUS		; Restore STATUS register - restores bank
		swapf	w_temp,F
		swapf	w_temp,W	; Restore W register
		retfie



;program initialisation

START		
		BANKSEL ADCON1			;disable analogue inputs
		movlw 	0x06
		movwf 	ADCON1
		BANKSEL PORTA
		bsf 	STATUS,		RP0	;select bank 1
		movlw	b'00000000'		;Set port data directions, data output
		movwf	ROW_TRIS
		movwf	COL_TRIS
		bcf 	STATUS,		RP0	;select bank 0

		clrf	COL_PORT		;turn OFF all LED's
		movlw	0xFF
		movwf	ROW_PORT	

		movlw	0x01
		movwf	row_pos

		call	Clear			;clear display registers

		;	Set up Timer 2.
	
		;movlw	b'01111110'		; Post scale /16, pre scale /16, TMR2 ON
		;uncomment previous line, and comment next line, to slow multiplexing speed
		;so you can see the multiplexing happening
		movlw	b'00010110'		; Post scale /4, pre scale /16, TMR2 ON
		movwf	T2CON

		bsf 	STATUS,		RP0	;select bank 1

		movlw	.249			; Set up comparator
		movwf	PR2

		bsf	PIE1,TMR2IE		; Enable TMR2 interrupt

		bcf 	STATUS,		RP0	;select bank 0

		; Global interrupt enable

		bsf	INTCON,PEIE		; Enable all peripheral interrupts
		bsf	INTCON,GIE		; Global interrupt enable

		bcf 	STATUS,		RP0	;select bank 0
	

;main program loop

		
Main		call	Clear
		clrf	offsetL1		;set offset to zero
		clrf	offsetH1
Message		
		call	String_Table		;get a character from the string table
		xorlw	0x00			;is it a zero? - if so end of string
		btfsc	STATUS, Z
		goto	Main
		movwf	tmp2
		movlw	0x20			;subtract to get character index value
		subwf	tmp2, w
		call	Load_Char
		call	ScrollL8
		incf	offsetL1, f		;increment 16 bit pointer to string
		btfsc   STATUS    , Z
		incf	offsetH1, f
		goto	Message



;display characters start at 0x00 for 'space' (rather than 0x20), and
;carry on from there the same as LCD text modules, ending with the right and
;left arrows.

;because each character takes 8 bytes of storage, we multiply the index value by 8

Load_Char	movwf	offsetL
		clrf	offsetH
		bcf	STATUS, C		;clear carry
		rlf	offsetL, f		;multiply index value by 8
		rlf	offsetH, f
		bcf	STATUS, C		;clear carry
		rlf	offsetL, f
		rlf	offsetH, f
		bcf	STATUS, C		;clear carry
		rlf	offsetL, f
		rlf	offsetH, f
		call	ASCII_Table
		movwf	zero1			
		incf	offsetL, f		;increment 16 bit pointer
		btfsc   STATUS    , Z
		incf	offsetH, f
		call	ASCII_Table
		movwf	one1
		incf	offsetL, f		;increment 16 bit pointer
		btfsc   STATUS    , Z
		incf	offsetH, f
		call	ASCII_Table
		movwf	two1
		incf	offsetL, f		;increment 16 bit pointer
		btfsc   STATUS    , Z
		incf	offsetH, f
		call	ASCII_Table
		movwf	three1
		incf	offsetL, f		;increment 16 bit pointer
		btfsc   STATUS    , Z
		incf	offsetH, f
		call	ASCII_Table
		movwf	four1
		incf	offsetL, f		;increment 16 bit pointer
		btfsc   STATUS    , Z
		incf	offsetH, f
		call	ASCII_Table
		movwf	five1
		incf	offsetL, f		;increment 16 bit pointer
		btfsc   STATUS    , Z
		incf	offsetH, f
		call	ASCII_Table
		movwf	six1
		incf	offsetL, f		;increment 16 bit pointer
		btfsc   STATUS    , Z
		incf	offsetH, f
		call	ASCII_Table
		movwf	seven1
		return

String_Table  					;perform read from long table
		movlw	High(String)
		addwf	offsetH1, W
		movwf	PCLATH
		movlw	Low(String)
		addwf	offsetL1, w
		btfsc   STATUS    , C
		incf	PCLATH, f
		MOVWF   PCL

ASCII_Table  					;perform read from long table
		movlw	High(Table)
		addwf	offsetH, W
		movwf	PCLATH
		movlw	Low(Table)
		addwf	offsetL, w
		btfsc   STATUS    , C
		incf	PCLATH, f
		MOVWF   PCL

;start of subroutines

Clear		clrf	zero			;clear data storage
		clrf	one
		clrf	two
		clrf	three
		clrf	four
		clrf	five
		clrf	six
		clrf	seven
		return

Square		movlw	0xFF
		movwf	zero
		movlw	0x81
		movwf	one
		movlw	0x81
		movwf	two
		movlw	0x81
		movwf	three
		movlw	0x81
		movwf	four
		movlw	0x81
		movwf	five
		movlw	0x81
		movwf	six
		movlw	0xFF
		movwf	seven
		return

Block		movlw	0xFF
		movwf	zero1
		movlw	0xFF
		movwf	one1
		movlw	0xFF
		movwf	two1
		movlw	0xFF
		movwf	three1
		movlw	0xFF
		movwf	four1
		movlw	0xFF
		movwf	five1
		movlw	0xFF
		movwf	six1
		movlw	0xFF
		movwf	seven1
		return


ScrollR8	call	ScrollR
		call	ScrollDelay	
		call	ScrollR
		call	ScrollDelay
		call	ScrollR
		call	ScrollDelay
		call	ScrollR
		call	ScrollDelay	
		call	ScrollR
		call	ScrollDelay
		call	ScrollR
		call	ScrollDelay
		call	ScrollR
		call	ScrollDelay
		call	ScrollR
		call	ScrollDelay
		return

ScrollL8	call	ScrollL
		call	ScrollDelay
		call	ScrollL
		call	ScrollDelay	
		call	ScrollL
		call	ScrollDelay
		call	ScrollL
		call	ScrollDelay
		call	ScrollL
		call	ScrollDelay
		call	ScrollL
		call	ScrollDelay
		call	ScrollL
		call	ScrollDelay
		call	ScrollL
		call	ScrollDelay
		return

ScrollU8	call	ScrollUp
		call	ScrollDelay
		call	ScrollUp
		call	ScrollDelay
		call	ScrollUp
		call	ScrollDelay
		call	ScrollUp
		call	ScrollDelay
		call	ScrollUp
		call	ScrollDelay	
		call	ScrollUp
		call	ScrollDelay
		call	ScrollUp
		call	ScrollDelay
		call	ScrollUp
		call	ScrollDelay
		return

ScrollD8	call	ScrollDn
		call	ScrollDelay
		call	ScrollDn
		call	ScrollDelay
		call	ScrollDn
		call	ScrollDelay
		call	ScrollDn
		call	ScrollDelay
		call	ScrollDn
		call	ScrollDelay
		call	ScrollDn
		call	ScrollDelay
		call	ScrollDn
		call	ScrollDelay
		call	ScrollDn
		call	ScrollDelay
		return

ScrollDelay	call	Delay100
		return

ScrollUp	movf	six, w
		movwf	seven
		movf	five, w
		movwf	six
		movf	four, w
		movwf	five
		movf	three, w
		movwf	four
		movf	two, w
		movwf	three
		movf	one, w
		movwf	two
		movf	zero, w
		movwf	one
		movf	seven1, w
		movwf	zero
		movf	six1, w
		movwf	seven1
		movf	five1, w
		movwf	six1
		movf	four1, w
		movwf	five1
		movf	three1, w
		movwf	four1
		movf	two1, w
		movwf	three1
		movf	one1, w
		movwf	two1
		movf	zero1, w
		movwf	one1
		movlw	0x00
		movwf	zero1
		return

ScrollDn	movf	one, w
		movwf	zero
		movf	two, w
		movwf	one
		movf	three, w
		movwf	two
		movf	four, w
		movwf	three
		movf	five, w
		movwf	four
		movf	six, w
		movwf	five
		movf	seven, w
		movwf	six
		movf	zero1, w
		movwf	seven
		movf	one1, w
		movwf	zero1
		movf	two1, w
		movwf	one1
		movf	three1, w
		movwf	two1
		movf	four1, w
		movwf	three1
		movf	five1, w
		movwf	four1
		movf	six1, w
		movwf	five1
		movf	seven1, w
		movwf	six1
		movlw	0x00
		movwf	seven1
		return

ScrollL		bcf	STATUS, C
		rlf	zero1, f
		rlf	zero, f
		bcf	STATUS, C
		rlf	one1, f
		rlf	one, f
		bcf	STATUS, C
		rlf	two1, f
		rlf	two, f
		bcf	STATUS, C
		rlf	three1, f
		rlf	three, f
		bcf	STATUS, C
		rlf	four1, f
		rlf	four, f
		bcf	STATUS, C
		rlf	five1, f
		rlf	five, f
		bcf	STATUS, C
		rlf	six1, f
		rlf	six, f
		bcf	STATUS, C
		rlf	seven1, f
		rlf	seven, f
		return

ScrollR		bcf	STATUS, C
		rrf	zero1, f
		rrf	zero, f
		bcf	STATUS, C
		rrf	one1, f
		rrf	one, f
		bcf	STATUS, C
		rrf	two1, f
		rrf	two, f
		bcf	STATUS, C
		rrf	three1, f
		rrf	three, f
		bcf	STATUS, C
		rrf	four1, f
		rrf	four, f
		bcf	STATUS, C
		rrf	five1, f
		rrf	five, f
		bcf	STATUS, C
		rrf	six1, f
		rrf	six, f
		bcf	STATUS, C
		rrf	seven1, f
		rrf	seven, f
		return

Invert		movlw	0xFF
		xorwf	zero, f
		xorwf	one, f
		xorwf	two, f
		xorwf	three, f
		xorwf	four, f
		xorwf	five, f
		xorwf	six, f
		xorwf	seven, f
		return

Transfer	movf	zero1, w
		movwf	zero
		movf	one1, w
		movwf	one
		movf	two1, w
		movwf	two
		movf	three1, w
		movwf	three
		movf	four1, w
		movwf	four
		movf	five1, w
		movwf	five
		movf	six1, w
		movwf	six
		movf	seven1, w
		movwf	seven
		return

Delay1Sec	call	Delay250
		call	Delay250
		call	Delay250
		call	Delay250
		Return


Delay255	movlw	0xff	;delay 255mS
		goto	d0
Delay250	movlw	d'250'	;delay 250mS
		goto	d0
Delay100	movlw	d'100'	;delay 100mS
		goto	d0
Delay50		movlw	d'50'	;delay 50mS
		goto	d0
Delay20		movlw	d'20'	;delay 20mS
		goto	d0
Delay10		movlw	d'10'	;delay 10mS
		goto	d0
Delay1		movlw	d'1'	;delay 1mS
		goto	d0
Delay5		movlw	0x05	;delay 5ms
d0		movwf	count1
d1		movlw	0xE7
		movwf	counta
		movlw	0x04
		movwf	countb
Delay_0		decfsz	counta, f
		goto	$+2
		decfsz	countb, f
		goto	Delay_0
		decfsz	count1	,f
		goto	d1
		retlw	0x00


Table		
		RETLW	b'00000000'		;SPACE
            	RETLW   b'0000000'
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;!
            	RETLW   b'00010000'
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'

		RETLW	b'00000000'		;"
            	RETLW   b'00000000'
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00101000'		
            	RETLW   b'00101000'		
            	RETLW   b'00101000'

		RETLW	b'00000000'		;#
            	RETLW   b'00101000'
            	RETLW   b'00101000'		
            	RETLW   b'01111100'		
            	RETLW   b'00101000'		
            	RETLW   b'01111100'		
            	RETLW   b'00101000'		
            	RETLW   b'00101000'

		RETLW	b'00000000'		;$
            	RETLW   b'00010000'
            	RETLW   b'01111000'		
            	RETLW   b'00010100'		
            	RETLW   b'00111000'		
            	RETLW   b'01010000'		
            	RETLW   b'00111100'		
            	RETLW   b'00010000'

		RETLW	b'00000000'		;%
            	RETLW   b'00001100'
            	RETLW   b'01001100'		
            	RETLW   b'00100000'		
            	RETLW   b'00010000'		
            	RETLW   b'00001000'		
            	RETLW   b'01100100'		
            	RETLW   b'01100000'

		RETLW	b'00000000'		;&
            	RETLW   b'00110100'
            	RETLW   b'01001000'		
            	RETLW   b'01010100'		
            	RETLW   b'00100000'		
            	RETLW   b'01010000'		
            	RETLW   b'01001000'		
            	RETLW   b'00110000'

		RETLW	b'00000000'		;'
            	RETLW   b'00000000'
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00100000'		
            	RETLW   b'00010000'		
            	RETLW   b'00110000'

		RETLW	b'00000000'		;(
            	RETLW   b'00001000'
            	RETLW   b'00010000'		
            	RETLW   b'00100000'		
            	RETLW   b'00100000'		
            	RETLW   b'00100000'		
            	RETLW   b'00010000'		
            	RETLW   b'00001000'

		RETLW	b'00000000'		;)
            	RETLW   b'00100000'
            	RETLW   b'00010000'		
            	RETLW   b'00001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00010000'		
            	RETLW   b'00100000'

		RETLW	b'00000000'		;*
            	RETLW   b'00010000'
            	RETLW   b'01010100'		
            	RETLW   b'00111000'		
            	RETLW   b'01010100'		
            	RETLW   b'00010000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;+
            	RETLW   b'00000000'
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'01111100'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;,
            	RETLW   b'00100000'
            	RETLW   b'00010000'		
            	RETLW   b'00110000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;-
            	RETLW   b'00000000'
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'01111100'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;.
            	RETLW   b'00110000'
            	RETLW   b'00110000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;/
            	RETLW   b'00000000'
            	RETLW   b'01000000'		
            	RETLW   b'00100000'		
            	RETLW   b'00010000'		
            	RETLW   b'00001000'		
            	RETLW   b'00000100'		
            	RETLW   b'00000000'		

		RETLW	0x00			;0
		RETLW	0x38
		RETLW	0x44
		RETLW	0x4C
		RETLW	0x54
		RETLW	0x64
		RETLW	0x44
		RETLW	0x38

		RETLW	0x00			;1
		RETLW	0x38
		RETLW	0x10
		RETLW	0x10
		RETLW	0x10
		RETLW	0x10
		RETLW	0x30
		RETLW	0x10


		RETLW	0x00			;2
		RETLW	0x7C
		RETLW	0x20
		RETLW	0x10
		RETLW	0x08
		RETLW	0x04
		RETLW	0x44
		RETLW	0x38

		RETLW	0x00			;3
		RETLW	0x38
		RETLW	0x44
		RETLW	0x04
		RETLW	0x08
		RETLW	0x10
		RETLW	0x08
		RETLW	0x7C

		RETLW	0x00			;4
		RETLW	0x08
		RETLW	0x08
		RETLW	0x7C
		RETLW	0x48
		RETLW	0x28
		RETLW	0x18
		RETLW	0x08

		RETLW	0x00			;5
		RETLW	0x38
		RETLW	0x44
		RETLW	0x04
		RETLW	0x04
		RETLW	0x78
		RETLW	0x40
		RETLW	0x7C

		RETLW	0x00			;6
		RETLW	0x38
		RETLW	0x44
		RETLW	0x44
		RETLW	0x78
		RETLW	0x40
		RETLW	0x20
		RETLW	0x18

		RETLW	0x00			;7
		RETLW	0x20
		RETLW	0x20
		RETLW	0x20
		RETLW	0x10
		RETLW	0x08
		RETLW	0x04
		RETLW	0x7C

		RETLW	0x00			;8
		RETLW	0x38
		RETLW	0x44
		RETLW	0x44
		RETLW	0x38
		RETLW	0x44
		RETLW	0x44
		RETLW	0x38

		RETLW	0x00			;9
		RETLW	0x30
		RETLW	0x08
		RETLW	0x04
		RETLW	0x3C
		RETLW	0x44
		RETLW	0x44
		RETLW	0x38

		RETLW	b'00000000'		;:
            	RETLW   b'00000000'
            	RETLW   b'00110000'
            	RETLW   b'00110000'		
            	RETLW   b'00000000'		
            	RETLW   b'00110000'		
            	RETLW   b'00110000'		
            	RETLW   b'00000000'		


		RETLW	b'00000000'		;;
            	RETLW   b'00100000'
            	RETLW   b'00010000'
            	RETLW   b'00110000'		
            	RETLW   b'00000000'		
            	RETLW   b'00110000'		
            	RETLW   b'00110000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;<
            	RETLW   b'00000100'
            	RETLW   b'00001000'
            	RETLW   b'00010000'		
            	RETLW   b'00100000'		
            	RETLW   b'00010000'		
            	RETLW   b'00001000'		
            	RETLW   b'00000100'

		RETLW	b'00000000'		;=
            	RETLW   b'00000000'
            	RETLW   b'00000000'
            	RETLW   b'01111100'		
            	RETLW   b'00000000'		
            	RETLW   b'01111100'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;>
            	RETLW   b'01000000'
            	RETLW   b'00100000'
            	RETLW   b'00010000'		
            	RETLW   b'00001000'		
            	RETLW   b'00010000'		
            	RETLW   b'00100000'		
            	RETLW   b'01000000'	

		RETLW	b'00000000'		;?
            	RETLW   b'00010000'
            	RETLW   b'00000000'
            	RETLW   b'00010000'		
            	RETLW   b'00001000'		
            	RETLW   b'00000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00111000'
	

		RETLW	b'00000000'		;@
            	RETLW   b'00111000'
            	RETLW   b'01010100'
            	RETLW   b'01010100'		
            	RETLW   b'00110100'		
            	RETLW   b'00000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00111000'

		RETLW	b'00000000'		;A
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01111100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00111000'

		RETLW	b'00000000'		;B
            	RETLW   b'01111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01111000'
		
		RETLW	b'00000000'		;C
            	RETLW   b'00111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000100'		
            	RETLW   b'00111000'

		RETLW	b'00000000'		;D
            	RETLW   b'01110000'		
            	RETLW   b'01001000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01001000'		
            	RETLW   b'01110000'

		RETLW	b'00000000'		;E
            	RETLW   b'01111100'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01111000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01111100'

		RETLW	b'00000000'		;F
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01111000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01111100'

		RETLW	b'00000000'		;G
            	RETLW   b'00111100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01011100'		
            	RETLW   b'01000000'		
            	RETLW   b'01000100'		
            	RETLW   b'00111000'

		RETLW	b'00000000'		;H
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01111100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'

		RETLW	b'00000000'		;I
            	RETLW   b'00111000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00111000'

		RETLW	b'00000000'		;J
            	RETLW   b'00110000'		
            	RETLW   b'01001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00011100'

		RETLW	b'00000000'		;K
            	RETLW   b'01000100'		
            	RETLW   b'01001000'		
            	RETLW   b'01010000'		
            	RETLW   b'01100000'		
            	RETLW   b'01010000'		
            	RETLW   b'01001000'		
            	RETLW   b'01000100'

		RETLW	b'00000000'		;L
            	RETLW   b'01111100'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'

		RETLW	b'00000000'		;M
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01010100'		
            	RETLW   b'01010100'		
            	RETLW   b'01101100'		
            	RETLW   b'01000100'

		RETLW	b'00000000'		;N
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01001100'		
            	RETLW   b'01010100'		
            	RETLW   b'01100100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'

		RETLW	b'00000000'		;O
            	RETLW   b'00111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00111000'

		RETLW	b'00000000'		;P
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01111000'

		RETLW	b'00000000'		;Q
            	RETLW   b'00110100'		
            	RETLW   b'01001000'		
            	RETLW   b'01010100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00111000'

		RETLW	b'00000000'		;R
            	RETLW   b'01000100'		
            	RETLW   b'01001000'		
            	RETLW   b'01010000'		
            	RETLW   b'01111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00111000'

		RETLW	b'00000000'		;S
            	RETLW   b'01111000'		
            	RETLW   b'00000100'		
            	RETLW   b'00000100'		
            	RETLW   b'00111000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'00111100'

		RETLW	b'00000000'		;T
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'01111100'

		RETLW	b'00000000'		;U
            	RETLW   b'00111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'

		RETLW	b'00000000'		;V
            	RETLW   b'00010000'		
            	RETLW   b'00101000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'

		RETLW	b'00000000'		;W
            	RETLW   b'00101000'		
            	RETLW   b'01010100'		
            	RETLW   b'01010100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'

		RETLW	b'00000000'		;X
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00101000'		
            	RETLW   b'00010000'		
            	RETLW   b'00101000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'

		RETLW	b'00000000'		;Y
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00101000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'

		RETLW	b'00000000'		;Z
            	RETLW   b'01111100'		
            	RETLW   b'01000000'		
            	RETLW   b'00100000'		
            	RETLW   b'00010000'		
            	RETLW   b'00001000'		
            	RETLW   b'00000100'		
            	RETLW   b'01111100'


		RETLW	b'00000000'		;[
            	RETLW   b'00111000'
            	RETLW   b'00100000'
            	RETLW   b'00100000'		
            	RETLW   b'00100000'		
            	RETLW   b'00100000'		
            	RETLW   b'00100000'		
            	RETLW   b'00111000'


           	RETLW   b'00000000'		;???
		RETLW	b'00010000'		
            	RETLW   b'00010000'
            	RETLW   b'01111100'
            	RETLW   b'00010000'		
            	RETLW   b'01111100'		
            	RETLW   b'00101000'		
            	RETLW   b'01000100'		
 


		RETLW	b'00000000'		;]
            	RETLW   b'00111000'
            	RETLW   b'00001000'
            	RETLW   b'00001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00111000'


		RETLW	b'00000000'		;^
            	RETLW   b'00000000'
            	RETLW   b'00000000'
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'01000100'		
            	RETLW   b'00101000'		
            	RETLW   b'00010000'

		RETLW	b'00000000'		;_
            	RETLW   b'01111100'
            	RETLW   b'00000000'
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;`
            	RETLW   b'00000000'
            	RETLW   b'00000000'
            	RETLW   b'00000000'		
            	RETLW   b'00000000'		
            	RETLW   b'00001000'		
            	RETLW   b'00010000'		
            	RETLW   b'00100000'

		RETLW	b'00000000'		;a
            	RETLW   b'00111100'		
            	RETLW   b'01000100'		
            	RETLW   b'00111100'		
            	RETLW   b'00000100'		
            	RETLW   b'00111000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;b
            	RETLW   b'01111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01100100'		
            	RETLW   b'01011000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'

		RETLW	b'00000000'		;c
            	RETLW   b'00111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'00111000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;d
            	RETLW   b'00111100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01001100'		
            	RETLW   b'00110100'		
            	RETLW   b'00000100'		
            	RETLW   b'00000100'

		RETLW	b'00000000'		;e
            	RETLW   b'00111000'		
            	RETLW   b'01000000'		
            	RETLW   b'01111100'		
            	RETLW   b'01000100'		
            	RETLW   b'00111000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;f
            	RETLW   b'00100000'		
            	RETLW   b'00100000'		
            	RETLW   b'00100000'		
            	RETLW   b'01110000'		
            	RETLW   b'00100000'		
            	RETLW   b'00100100'		
            	RETLW   b'00011000'

		RETLW	b'00111000'		;g
            	RETLW   b'00000100'		
            	RETLW   b'00000100'		
            	RETLW   b'00111100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00111100'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;h
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01100100'		
            	RETLW   b'01011000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'


		RETLW	b'00000000'		;i
            	RETLW   b'00111000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00110000'		
            	RETLW   b'00000000'		
            	RETLW   b'00010000'

		RETLW	b'00110000'		;j
            	RETLW   b'01001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00001000'		
            	RETLW   b'00011000'		
            	RETLW   b'00000000'		
            	RETLW   b'00001000'

		RETLW	b'00000000'		;k
            	RETLW   b'01001000'		
            	RETLW   b'01010000'		
            	RETLW   b'01100000'		
            	RETLW   b'01010000'		
            	RETLW   b'01001000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'

		RETLW	b'00000000'		;l
            	RETLW   b'00111000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00110000'


		RETLW	b'00000000'		;m
            	RETLW   b'01010100'		
            	RETLW   b'01010100'		
            	RETLW   b'01010100'		
            	RETLW   b'01010100'		
            	RETLW   b'01111000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'


		RETLW	b'00000000'		;n
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01100100'		
            	RETLW   b'01011000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'


		RETLW	b'00000000'		;o
            	RETLW   b'00111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00111000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'


		RETLW	b'01000000'		;p
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01111000'		
            	RETLW   b'01000100'		
            	RETLW   b'01111000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'


		RETLW	b'00000100'		;q
            	RETLW   b'00000100'		
            	RETLW   b'00000100'		
            	RETLW   b'00111100'		
            	RETLW   b'01001100'		
            	RETLW   b'00110100'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'


		RETLW	b'00000000'		;r
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01000000'		
            	RETLW   b'01100100'		
            	RETLW   b'01011000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'


		RETLW	b'00000000'		;s
            	RETLW   b'01111000'		
            	RETLW   b'00000100'		
            	RETLW   b'00111000'		
            	RETLW   b'01000000'		
            	RETLW   b'00111000'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'


		RETLW	b'00000000'		;t
            	RETLW   b'00011000'		
            	RETLW   b'00100100'		
            	RETLW   b'00100000'		
            	RETLW   b'00100000'		
            	RETLW   b'01110000'		
            	RETLW   b'00100000'		
            	RETLW   b'00100000'

		RETLW	b'00000000'		;u
            	RETLW   b'00110100'		
            	RETLW   b'01001100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;v
            	RETLW   b'00010000'		
            	RETLW   b'00101000'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;w
            	RETLW   b'00101000'		
            	RETLW   b'01010100'		
            	RETLW   b'01010100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;x
            	RETLW   b'01000100'		
            	RETLW   b'00101000'		
            	RETLW   b'00010000'		
            	RETLW   b'00101000'		
            	RETLW   b'01000100'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00111000'		;y
            	RETLW   b'00000100'		
            	RETLW   b'00000100'		
            	RETLW   b'00111100'		
            	RETLW   b'01000100'		
            	RETLW   b'01000100'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;Z
            	RETLW   b'01111100'
            	RETLW   b'00100000'
            	RETLW   b'00010000'		
            	RETLW   b'00001000'		
            	RETLW   b'01111100'		
            	RETLW   b'00000000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;(
            	RETLW   b'00001000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00100000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00001000'

		RETLW	b'00000000'		;|
            	RETLW   b'00010000'
            	RETLW   b'00010000'
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'

            	RETLW   b'00000000'		;)
		RETLW	b'00100000'		
            	RETLW   b'00010000'
            	RETLW   b'00010000'
            	RETLW   b'00001000'		
            	RETLW   b'00010000'		
            	RETLW   b'00010000'		
            	RETLW   b'00100000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;right arrow
            	RETLW   b'00000000'
            	RETLW   b'00010000'
            	RETLW   b'00001000'		
            	RETLW   b'01111100'		
            	RETLW   b'00001000'		
            	RETLW   b'00010000'		
            	RETLW   b'00000000'

		RETLW	b'00000000'		;left arrow
            	RETLW   b'00000000'
            	RETLW   b'00010000'
            	RETLW   b'00100000'		
            	RETLW   b'01111100'		
            	RETLW   b'00100000'		
            	RETLW   b'00010000'		
            	RETLW   b'00000000'

String		
            	RETLW   'W'
            	RETLW   'i'
            	RETLW   'n'
            	RETLW   'P'
            	RETLW   'i'
            	RETLW   'c'
            	RETLW   'P'
            	RETLW   'r'
            	RETLW   'o'
            	RETLW   'g'
            	RETLW   ' '
            	RETLW   '-'
            	RETLW   ' '
            	RETLW   'T'
            	RETLW   'u'
            	RETLW   't'
            	RETLW   'o'
            	RETLW   'r'
            	RETLW   'i'
            	RETLW   'a'
            	RETLW   'l'
            	RETLW   's'
            	RETLW   '.'
            	RETLW   0x00

	END
