#include <pic.h>				// pic specific identifiers
#define _XTAL_FREQ  20000000	// Xtal speed
__CONFIG(0x37F2);				// Config bits

#define ROW_PORT		PORTB			//
#define ROW_TRIS		TRISB
#define COL_PORT		PORTC			//
#define COL_TRIS		TRISC

unsigned char shape[] = {
	0xff,0x81,0x81,0x81,0x81,0x81,0x81,0xff, // square
	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff, // block
	0x00,0x38,0x44,0x4c,0x54,0x64,0x44,0x38, // 0
	0x00,0x38,0x10,0x10,0x10,0x10,0x30,0x10, // 1
	0x00,0x7c,0x20,0x10,0x08,0x04,0x44,0x38, // 2
	0x00,0x38,0x44,0x04,0x08,0x10,0x08,0x7c, // 3
	0x00,0x08,0x08,0x7c,0x48,0x28,0x18,0x08, // 4
	0x00,0x38,0x44,0x04,0x04,0x78,0x40,0x7c, // 5
	0x00,0x38,0x44,0x44,0x78,0x40,0x20,0x18, // 6
	0x00,0x20,0x20,0x20,0x10,0x08,0x04,0x7c, // 7
	0x00,0x38,0x44,0x44,0x38,0x44,0x44,0x38, // 8
	0x00,0x30,0x08,0x04,0x3c,0x44,0x44,0x38}; // 9

	// Required prototypes
void DelayMs(int x), scroll(char dir), invert(void);
void clear(void);

	// Variables	
unsigned char pos;				// keep tabs on rows
unsigned char leds[8];			// led buffer
unsigned char leds2[8];			// 2nd led buffer
unsigned char pow[] = {1,2,4,8,16,32,64,128}; // power array	

void main(void)							// program entry
	{
	char x,y,z,t;
	ADCON1 = 0x6;
	ROW_TRIS = 0b00000000;				// row port as outputs
	COL_TRIS = 0b00000000;				// col port as outputs
	T2CON = 0x16;						// setup timer 2
	ROW_PORT = 0;
	COL_PORT = 0xFF;					// setup led array
	PR2 = 249;							// timer preload value
	TMR2IE = 1;							// enable timer 2 interrupt
	PEIE = 1;							// enable peripheral interrupt
	GIE = 1;							// enableglobal interrupt
	pos = 0;							// start at beginning
	while(1)							// endless Loop
		{
		for(y=0;y<4;y++)
			{
			z=0;
			for(x=0;x<10;x++)
				{
				t = (x*8)+17;
				for(z=0;z<8;z++)
					leds2[z] = shape[t+z];
				for(z=0;z<8;z++)
					scroll(y);
				}
			clear();
			DelayMs(1000);
			}
		}
	}

void interrupt ISR()
	{
	if(TMR2IF)			// omly do if timeout occured (1ms)
		{
		ROW_PORT = 0xff;
		COL_PORT = leds[pos];			// Retrive current array
		ROW_PORT -= pow[pos];			// Keep powers in table
		if(pos++ == 7) pos = 0;			// cycle through array buffer
		TMR2IF = 0;						// clear interrupt
		}	
	}

void clear()
	{
	char x;
	for(x=0;x<8;x++)		// clear buffer array
		leds[x] = 0;
	
	}
void scroll(char dir)
	{
	char x;
	switch (dir)
		{
		case 0:
			for(x=0;x<8;x++)
				{
				if(leds2[x] & 1)		// scroll buffer 2
					leds[x]+=128;		// into buffer 1
				leds[x] >>= 1;			// moving left
				leds2[x]>>= 1;			//
				}
			DelayMs(100);
			break;				
		case 1:
			for(x=0;x<8;x++)
				{
				if(leds2[x] & 128)		// scroll the other way
					leds[x]+=1;			// moving right
				leds[x] <<= 1;
				leds2[x] <<= 1;
				}
			DelayMs(100);
			break;				
		case 2:
			for(x=7;x>0;x--)
				leds[x] = leds[x-1];	// more care needed		
			leds[0] = leds2[7];			// move led buffers up
			for(x=7;x>0;x--)			// from buffer 1
				leds2[x] = leds2[x-1];	// to buffer 1
			leds2[0] = 0;
			DelayMs(100);				// display time
			break;				
		case 3:
			for(x=0;x<8;x++)
				leds[x] = leds[x+1];	// move down wards
			leds2[7] = leds[0];
			for(x=0;x<8;x++)
				leds2[x] = leds2[x+1];	// same as before
			leds2[7] = 0;
			DelayMs(100);
		}

	}

void DelayMs(int x)
	{	// wrapper function for larger delays.
	while(x--)
		__delay_ms(1);
	}